@@ -1,5 +1,6 @@
 """vobject module for reading vCard and vCalendar files."""
 
+import copy
 import re
 import sys
 import logging
@@ -56,6 +57,12 @@ class VBase(object):
         self.parentBehavior = None
         self.isNative = False
     
+    def copy(self, copyit):
+        self.group = copyit.group
+        self.behavior = copyit.behavior
+        self.parentBehavior = copyit.parentBehavior
+        self.isNative = copyit.isNative
+        
     def validate(self, *args, **kwds):
         """Call the behavior's validate method, or return True."""
         if self.behavior:
@@ -87,6 +94,10 @@ class VBase(object):
                     self.setBehavior(behavior, cascade)
                     if isinstance(self, ContentLine) and self.encoded:
                         self.behavior.decode(self)
+            elif isinstance(self, ContentLine):
+                self.behavior = parentBehavior.defaultBehavior   
+                if self.encoded and self.behavior:
+                    self.behavior.decode(self)
 
     def setBehavior(self, behavior, cascade=True):
         """Set behavior. If cascade is True, autoBehavior all descendants."""
@@ -250,6 +261,21 @@ class ContentLine(VBase):
         if qp:
             self.value = str(self.value).decode('quoted-printable')
 
+    @classmethod
+    def duplicate(clz, copyit):
+        newcopy = clz('', {}, '')
+        newcopy.copy(copyit)
+        return newcopy
+
+    def copy(self, copyit):
+        super(ContentLine, self).copy(copyit)
+        self.name = copyit.name
+        self.value = copy.copy(copyit.value)
+        self.encoded = self.encoded
+        self.params = copy.copy(copyit.params)
+        self.singletonparams = copy.copy(copyit.singletonparams)
+        self.lineNumber = copyit.lineNumber
+        
     def __eq__(self, other):
         try:
             return (self.name == other.name) and (self.params == other.params) and (self.value == other.value)
@@ -361,6 +387,27 @@ class Component(VBase):
         
         self.autoBehavior()
 
+    @classmethod
+    def duplicate(clz, copyit):
+        newcopy = clz()
+        newcopy.copy(copyit)
+        return newcopy
+
+    def copy(self, copyit):
+        super(Component, self).copy(copyit)
+        
+        # deep copy of contents
+        self.contents = {}
+        for key, lvalue in copyit.contents.items():
+            newvalue = []
+            for value in lvalue:
+                newitem = value.duplicate(value)
+                newvalue.append(newitem)
+            self.contents[key] = newvalue
+
+        self.name = copyit.name
+        self.useBegin = copyit.useBegin
+         
     def setProfile(self, name):
         """Assign a PROFILE to this unnamed component.
         
@@ -457,6 +504,9 @@ class Component(VBase):
                 obj = obj.transformToNative()     
             except (KeyError, AttributeError):
                 obj = ContentLine(objOrName, [], '', group)
+            if obj.behavior is None and self.behavior is not None:
+                if isinstance(obj, ContentLine):
+                    obj.behavior = self.behavior.defaultBehavior
         self.contents.setdefault(obj.name.lower(), []).append(obj)
         return obj
 
@@ -713,20 +763,23 @@ def getLogicalLines(fp, allowQP=True, findBegin=False):
     """
     if not allowQP:
         bytes = fp.read(-1)
-        if type(bytes[0]) == unicode:
-            val = bytes
-        elif not findBegin:
-            val = bytes.decode('utf-8')
-        else:
-            for encoding in 'utf-8', 'utf-16-LE', 'utf-16-BE':
-                try:
-                    val = bytes.decode(encoding)
-                    if begin_re.search(val) is not None:
-                        break
-                except UnicodeDecodeError:
-                    pass
+        if len(bytes) > 0:
+            if type(bytes[0]) == unicode:
+                val = bytes
+            elif not findBegin:
+                val = bytes.decode('utf-8')
             else:
-                raise ParseError, 'Could not find BEGIN when trying to determine encoding'
+                for encoding in 'utf-8', 'utf-16-LE', 'utf-16-BE':
+                    try:
+                        val = bytes.decode(encoding)
+                        if begin_re.search(val) is not None:
+                            break
+                    except UnicodeDecodeError:
+                        pass
+                else:
+                    raise ParseError, 'Could not find BEGIN when trying to determine encoding'
+        else:
+            val = bytes
 
         lineNumber = 1
         for match in logical_lines_re.finditer(val):
@@ -871,7 +924,8 @@ class Stack:
     def push(self, obj): self.stack.append(obj)
     def pop(self): return self.stack.pop()
 
-def readComponents(streamOrString, validate=False, transform=True, findBegin=True):
+def readComponents(streamOrString, validate=False, transform=True,
+                   findBegin=True, ignoreUnreadable=False):
     """Generate one Component at a time from a stream.
 
     >>> import StringIO
@@ -891,7 +945,19 @@ def readComponents(streamOrString, validate=False, transform=True, findBegin=Tru
     versionLine = None
     n = 0
     for line, n in getLogicalLines(stream, False, findBegin):
-        vline = textLineToContentLine(line, n)
+        if ignoreUnreadable:
+            try:
+                vline = textLineToContentLine(line, n)
+            except VObjectError, e:
+                if e.lineNumber is not None:
+                    msg = "Skipped line %(lineNumber)s, message: %(msg)s"
+                else:
+                    msg = "Skipped a line, message: %(msg)s"
+                logger.error(msg % {'lineNumber' : e.lineNumber, 
+                                    'msg' : e.message})
+                continue
+        else:
+            vline = textLineToContentLine(line, n)
         if   vline.name == "VERSION":
             versionLine = vline
             stack.modifyTop(vline)
@@ -926,9 +992,11 @@ def readComponents(streamOrString, validate=False, transform=True, findBegin=Tru
         yield stack.pop()
 
 
-def readOne(stream, validate=False, transform=True, findBegin=True):
+def readOne(stream, validate=False, transform=True, findBegin=True,
+            ignoreUnreadable=False):
     """Return the first component from stream."""
-    return readComponents(stream, validate, transform, findBegin).next()
+    return readComponents(stream, validate, transform, findBegin,
+                          ignoreUnreadable).next()
 
 #--------------------------- version registry ----------------------------------
 __behaviorRegistry={}
